@@ -64,7 +64,7 @@
                                 study_id = NULL,
                                 molecular_profile_id = NULL,
                                 sample_study_pairs = NULL,
-                                data_type = c("mutation", "cna", "fusion", "structural_variant"),
+                                data_type = c("mutation", "cna", "fusion", "structural_variant", "segment"),
 
                                 genes = NULL,
                                 panel = NULL,
@@ -87,16 +87,20 @@
   if(length(study_id) > 1 | length(molecular_profile_id) > 1) {
      cli::cli_abort("More than 1 {.code study_id} or {.code molecular_profile_id} was passed. Please use the {.code sample_study_pairs} argument instead")
   }
-  data_type <- match.arg(data_type) %>%
-    purrr::when(. ==  "structural_variant" ~ "fusion",
-                TRUE ~ .)
+
+  data_type <- match.arg(data_type)
+
+  data_type <- dplyr::case_when(
+    data_type == "structural_variant" ~ "fusion",
+    TRUE ~ data_type)
 
   # this has to go in query URL
   url_data_type <- switch(
     data_type,
     "mutation" = "mutations",
     "fusion" = "structural-variant",
-    "cna" = "discrete-copy-number")
+    "cna" = "discrete-copy-number",
+    "segment" = "copy-number-segments")
 
   # Make Informed guesses on parameters ---------------------------------------
 
@@ -111,20 +115,19 @@
 
 
     # Get study ID ---------
-    resolved_study_id <- study_id %>%
-      purrr::when(!is.null(.) ~ .,
-                  !is.null(molecular_profile_id) ~ .lookup_study_name(molecular_profile_id = molecular_profile_id,
-                                                                      study_id = .,
-                                                                      base_url = base_url),
-                  # if both NULL
-                  ~ suppressMessages(.guess_study_id(study_id, resolved_url)))
-
+    resolved_study_id <- study_id %||% {
+      if(is_not_null(molecular_profile_id)) {
+        .lookup_study_name(molecular_profile_id = molecular_profile_id,
+                           study_id = study_id,
+                           base_url = base_url)
+      } else {
+        suppressMessages(.guess_study_id(study_id, resolved_url))
+      }
+    }
 
     # Get molecular profile ID ---------
-    resolved_molecular_profile_id <- molecular_profile_id %>%
-      purrr::when(
-        !is.null(.) ~ .,
-        ~ .lookup_profile_name(data_type, study_id = resolved_study_id, base_url))
+    resolved_molecular_profile_id <- molecular_profile_id %||%
+      .lookup_profile_name(data_type, resolved_study_id, base_url = base_url)
 
 
 
@@ -145,13 +148,12 @@
   panel_genes <- .get_panel_entrez(panel_id = panel, base_url = base_url)
 
   # if genes arg passed, get gene IDs if panel, or entrez IDs if hugo
-  resolved_genes <- genes %>%
-    purrr::when(
-      is.character(.) ~ {
-#        cli::cli_inform("Hugo symbols were converted to entrez IDs in order to query the cBioPortal API (see {.code ?get_entrez_id} for more info)")
-        get_entrez_id(., base_url = base_url)$entrezGeneId
-      },
-      TRUE ~ .)
+  resolved_genes <-
+    if(is.character(genes)) {
+      get_entrez_id(genes, base_url = base_url)$entrezGeneId
+    } else {
+      genes
+    }
 
   resolved_genes <- c(panel_genes, resolved_genes) %>%
     unique()
@@ -162,7 +164,7 @@
   # If user passes study_id and data_type we can pull the correct molecular ID
   if(!("molecular_profile_id" %in% colnames(sample_study_pairs))) {
 
-    unique_study_id <- distinct(select(sample_study_pairs, .data$study_id)) %>%
+    unique_study_id <- distinct(select(sample_study_pairs, "study_id")) %>%
       mutate(molecular_profile_id =
                purrr::map(.data$study_id,
                            ~.lookup_profile_name(.x,
@@ -182,14 +184,14 @@
 
     sample_study_pairs_nest <- sample_study_pairs %>%
       group_by(.data$study_id, .data$molecular_profile_id) %>%
-      tidyr::nest(sample_id_nest = .data$sample_id) %>%
+      tidyr::nest(sample_id_nest = "sample_id") %>%
       mutate(url_path = paste0("molecular-profiles/",
                                .data$molecular_profile_id,
                                "/", url_data_type, "/fetch?")) %>%
       ungroup() %>%
-      select(.data$url_path, .data$sample_id_nest)
+      select("url_path", "sample_id_nest")
 
-    quer_res <- purrr::map2(
+    quer_res <- purrr::map2_dfr(
       sample_study_pairs_nest$url_path,
       sample_study_pairs_nest$sample_id_nest,
 
@@ -205,56 +207,78 @@
                        body = body_n, base_url = base_url)
 
 
-        bind_rows(res$content)
+        # if embedded list in API response (e.g. namespace columns)
+        if(length(res$content) > 0) {
+          result <- purrr::map_dfr(res$content, ~purrr::list_flatten(.x))
+        } else {
+          result <- NULL
+        }
 
       })
 
-    df <- quer_res %>% bind_rows()
+    df <- quer_res
 
   }
 
-  # * FUSIONS query ----------------------------------------------------------------------
+  # * FUSIONS/SEGMENTS query ----------------------------------------------------------------------
+
+  # POST: /structural-variant/fetch ---
+  # BODY: sampleMolecularIdentifiers: molecularProfileId, sampleId
 
   # Fusions endpoint works a little differently than Mut and CNA
   # Instead of passing a sample list, you pass individual sample IDs (retrieved using list)
   # Main Goal in this function is to return all results without specifying specific genes to query (as is needed in other endpoints).
 
-  if(data_type == "fusion") {
+  if(data_type %in% c("fusion", "segment")) {
 
-    quer_res <- purrr::map2(
-      sample_study_pairs$sample_id,
-      sample_study_pairs$molecular_profile_id,
+    quer_res <- purrr::pmap_dfr(
+      sample_study_pairs,
 
-      function(x, y) {
-
-
-        body_n <- list(
-          sampleMolecularIdentifiers = as.data.frame(list(
-            molecularProfileId = jsonlite::unbox(y),
-            sampleId = x
-          ))
-        )
+      function(sample_id, molecular_profile_id, study_id) {
 
+        body_n <- switch(data_type,
+                       fusion = {
+                         list(
+                           sampleMolecularIdentifiers = as.data.frame(list(
+                             molecularProfileId = jsonlite::unbox(molecular_profile_id),
+                             sampleId = sample_id
+                           ))
+                         )
+                       },
+                       segment = {
+                         data.frame(
+                           "sampleId" = sample_id,
+                           "studyId" = study_id)
+                       })
 
         res <- cbp_api(url_path = paste0(url_data_type, "/fetch?"),
                        method = "post",
                        body = body_n,
                        base_url = base_url)
 
-        res$content
+        # if embedded list in API response (e.g. namespace columns)
+        if(length(res$content) > 0) {
+          result <- purrr::map_dfr(res$content, ~purrr::list_flatten(.x))
+        } else {
+          result <- NULL
+        }
 
       })
 
-    df_fus <- quer_res %>% bind_rows()
-
-    df <- df_fus %>%
-      purrr::when(
-        (nrow(.) > 0 & !is.null(resolved_genes)) ~ filter(., (.data$site1EntrezGeneId %in% resolved_genes) | (.data$site2EntrezGeneId %in% resolved_genes)),
-        TRUE ~ .)
+    df_fus <- quer_res
 
     # Since you don't query by genes, filter genes at end so behaviour is consistent
     # with mutation/cna endpoints where you have to specify genes to query
-
+    df <- switch(data_type,
+      fusion = {
+        if (nrow(df_fus) > 0 & !is.null(resolved_genes)) {
+          filter(df_fus, (.data$site1EntrezGeneId %in% resolved_genes) | (.data$site2EntrezGeneId %in% resolved_genes))
+        } else {
+          df_fus
+        }
+      },
+      segment = df_fus
+    )
 
   }
 
@@ -266,6 +290,7 @@
     # Fusions already has hugo by default from API
     df <- switch(data_type,
                  "fusion" = df,
+                 "segment" = df,
                  "mutation" = if(nrow(df) > 0) .lookup_hugo(df, base_url = base_url),
                  "cna" = if(nrow(df) > 0) .lookup_hugo(df, base_url = base_url))
 
@@ -407,17 +432,55 @@ get_fusions_by_sample <- function(sample_id = NULL,
 get_structural_variants_by_sample <- get_fusions_by_sample
 
 
+
+#' Get Copy Number Segmentation Data By Sample ID
+#'
+#' @inheritParams .get_data_by_sample
+#' @return A dataframe of CNA segments
+#' @export
+#'
+#' @examples
+#' \dontrun{
+#' set_cbioportal_db("public")
+#'
+#' get_segments_by_sample(sample_id = c("s_C_CAUWT7_P001_d"),
+#'                  study_id = "prad_msk_2019")
+#'                  }
+
+
+get_segments_by_sample <- function(sample_id = NULL,
+                                  study_id = NULL,
+                                  sample_study_pairs = NULL,
+                                  base_url = NULL) {
+
+  .get_data_by_sample(sample_id = sample_id,
+                      study_id = study_id,
+                      molecular_profile_id = NULL,
+                      sample_study_pairs = sample_study_pairs,
+                      data_type = c("segment"),
+                      genes = NULL,
+                      panel = NULL,
+                      # this shouldn't matter for seg data
+                      add_hugo = TRUE,
+                      base_url = base_url)
+
+
+}
+
 #' Get All Genomic Information By Sample IDs
 #'
 #' @inheritParams .get_data_by_sample
-#' @return A list of mutations, cna and structural variants (including fusions), if available.
+#' @param return_segments Default is `FALSE` where copy number segmentation data won't be returned in addition to the mutation, cna and structural variant data.
+#' `TRUE` will return any available segmentation data with results.
+#' @return A list of mutations, cna and structural variants (including fusions), if available. Will also return copy number segmentation data if `return_segments = TRUE`.
 #' @export
 #'
 #'
 #' @examples
 #' \dontrun{
 #' get_genetics_by_sample(sample_id = c("TCGA-OR-A5J2-01","TCGA-OR-A5J6-01"),
-#'  study_id = "acc_tcga")
+#'  study_id = "acc_tcga",
+#'  return_segments = TRUE)
 #' }
 #
 get_genetics_by_sample <- function(sample_id = NULL,
@@ -426,11 +489,18 @@ get_genetics_by_sample <- function(sample_id = NULL,
                                    genes = NULL,
                                    panel = NULL,
                                    add_hugo = TRUE,
-                                   base_url = NULL) {
+                                   base_url = NULL,
+                                   return_segments = FALSE) {
+
+  data_types <- c("mutation", "cna", "structural_variant")
+
+  if(return_segments) {
+    data_types <- c(data_types, "segment")
+  }
 
   safe_get_data <- purrr::safely(.get_data_by_sample, quiet = TRUE)
 
-  res <-  c("mutation", "cna", "structural_variant") %>%
+  res <- data_types %>%
     purrr::set_names() %>%
     purrr::map(., function(x) {
       safe_get_data(sample_id = sample_id,
@@ -448,9 +518,9 @@ get_genetics_by_sample <- function(sample_id = NULL,
   errors <- purrr::compact(purrr::map(res, "error"))
 
   switch(!purrr::is_empty(errors),
-         purrr::imap(errors, ~cli_alert_warning(c("No {.val {.y}} data returned. Error:  ",
+         purrr::imap(errors, ~cli_alert_warning(c("No {.val { .y}} data returned. Error:  ",
                                                   # why no red :(
-                                                  cli::col_red('{.x$message}'))))
+                                                  cli::col_red('{ .x$message}'))))
   )
 
   genetics
